#<profile4xi profile = "02" /> 
# -------------------------------------------------------------------------------------------------
#<program4xi program = "socketlistener4xi.py">
#<abstract4xi abstract = "provides the class SocketListener4Xi"/>
#<refer4xi reference = "connection.py"/> 
#<state4xi status = "ACK4tagging; ACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
# -------------------------------------------------------------------------------------------------


# -------------------------------------------------------------------------------------------------
# Public API
# -------------------------------------------------------------------------------------------------
#<interface4xi interface = " 
__all__ = [ 'SocketListener4Xi' ] 
#"
#/>

# -------------------------------------------------------------------------------------------------
# Standard Imports                          
# -------------------------------------------------------------------------------------------------
#<standardize4xi standard = "
import errno
import io
import itertools
import os
import pickle
import select
import socket
import struct
import sys
import time
import tempfile
try:
    import _winapi
    from _winapi import WAIT_OBJECT_0, WAIT_TIMEOUT, INFINITE
except ImportError:
    if sys.platform == 'win32':
        raise
    _winapi = None 
#"
#/>

# -------------------------------------------------------------------------------------------------
# Multiprocessing Imports                          
# -------------------------------------------------------------------------------------------------
#<process4xi process = "
import _multiprocessing
from multiprocessing import connection
from multiprocessing.connection import Connection
from multiprocessing.connection import SocketListener
#"
#/>


# -------------------------------------------------------------------------------------------------
# DragonXi imports                               
# -------------------------------------------------------------------------------------------------
#<dragonize4xi dragonization = "
from ace_object4xi import ace_object4xi

#from connection4xi import Connection4Xi <-- do NOT use otherwise you end up to cyclic imports!

from constant_family4xi import get_af_inet4xi
from constant_family4xi import get_af_unix4xi
from constant_family4xi import get_af_pipe4xi

from duplicate4xi import duplicate4xi

from finalize4xi import Finalize4Xi

from print4xi import print4xi

from reason_object4xi import reason_object4xi

from sense_object4xi import sense_object4xi

from time4xi import time4xi
#"
#/>







# =================================================================================================
#<class4xi class = "SocketListener4Xi" base = "object" child = "None"
#   synopsis = "represents a socket which is bound to an address and listening" 
#   usage = "
#       listener4xi.py:
#           self._listener = SocketListener4Xi(address2xi, family2xi, backlog)
#"
#>
#<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
# =================================================================================================
class SocketListener4Xi(object):
    '''
    represents a socket which is bound to an address and listening
    '''
    
    # ---------------------------------------------------------------------------------------------
    # Public API of this class
    # ---------------------------------------------------------------------------------------------    
    #<expose4xi expose = "
    _exposed_ = ('close', 'accept',
                 'sense', 'reason', 'ace')          
    #"
    #/>
           
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "init-of-SocketListener4Xi" usage = "initializes SocketListener4Xi">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>    
    # ---------------------------------------------------------------------------------------------
    def __init__(self, address, family, backlog=1):
        '''
        initializes SocketListener4Xi
        ''' 
                          
        # -----------------------------------------------------------------------------------------    
        # (Step 1/9)
        # -----------------------------------------------------------------------------------------                 
                  
        class2print = "SocketListener4Xi"
        method2print = "__init__"
        
        message2print = "{(Step 1/9a) started" + "; init-of-SocketListener4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
                        
        message2print = "{(Step 1/9b) borning" + "; socketlistener2xi-of-SocketListener4Xi-borning: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)       
        
        message2print = "{(Step 1/9c) inheriting" + "; object-inheriting-by-socketlistener2xi-of-SocketListener4Xi: " + str(object)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
                
        message2print = "{(Step 1/9d) passed" + "; address-passed-to-init-of-SocketListener4Xi: " + str(address) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                          
        
        ack2address = False      #dragonized
        if address is None:

            message2print = "{(Step 1/9e) yielded" + "; address-yielded-by-init-of-SocketListener4Xi: " + str(address)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
        else:
            ack2address = True    

        message2print = "{(Step 1/9f) passed" + "; family-passed-to-init-of-SocketListener4Xi: " + str(family) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                          
                       

        message2print = "{(Step 1/9g) passed" + "; backlog-passed-to-init-of-SocketListener4Xi: " + str(backlog) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                          
                          
        # -----------------------------------------------------------------------------------------    
        # (Step 2/9)
        # -----------------------------------------------------------------------------------------                 
        self._socket = None
        ack2socket = False      #dragonized
        nack2socket = True
        try:   
            self._socket = socket.socket(getattr(socket, family))
            
            message2print = "{(Step 2/9a) set" + "; socket-of-socketlistener2xi-set-by-init-of-SocketListener4Xi to: " + str(self._socket)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            
            ack2socket = True
            
            message2print = "{(Step 2/9b) set" + "; ack2socket-set-by-init-of-SocketListener4Xi to: " + str(ack2socket)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            
               
        except:
            message2print = "{(Step 2/9c) yielded" + "; nack2socket-yielded-by-init-of-SocketListener4Xi: " + str(nack2socket)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
                
                          
        # -----------------------------------------------------------------------------------------    
        # (Step 3/9)
        # -----------------------------------------------------------------------------------------                 
        ack2bind = False
        nack2bind = True
        
        if ((ack2socket is True) and (ack2address is True)):                      
            message2print = "{(Step 3/9a) passing" + "; address-passing-to-socket-bind-of-socketlistener2xi-by-init-of-SocketListener4Xi: " + str(address) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
                    
            try:                
                self._socket.bind(address)
                
                message2print = "{(Step 3/9b) performed" + "; socket-bind-of-socketlistener2xi-performed-by-init-of-SocketListener4Xi: " + str(self._socket.bind) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                
                ack2bind = True
                
                message2print = "{(Step 3/9c) set" + "; ack2bind-set-by-init-of-SocketListener4Xi to: " + str(ack2bind)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
            except:
                message2print = "{(Step 3/9d) yielded" + "; nack2bind-yielded-by-init-of-SocketListener4Xi: " + str(nack2bind)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                 
                
                             
        else:
            message2print = "{(Step 3/9e) yielded" + "; nack2bind-yielded-by-init-of-SocketListener4Xi: " + str(nack2bind)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
                           
                          
        # -----------------------------------------------------------------------------------------    
        # (Step 4/9)
        # -----------------------------------------------------------------------------------------                 
        ack2listen = False
        nack2listen = True
        
        
        if ((ack2socket is True) and (ack2address is True)):                               
            message2print = "{(Step 4/9a) passing" + "; backlog-passing-to-socket-listen-of-socketlistener2xi-by-init-of-SocketListener4Xi: " + str(backlog) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
                    
            try:   
                self._socket.listen(backlog)
                
                message2print = "{(Step 4/9b) performed" + "; socket-listen-of-socketlistener2xi-performed-by-init-of-SocketListener4Xi: " + str(self._socket.listen) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                
                ack2listen = True 
                
                message2print = "{(Step 4/9c) set" + "; ack2listen-set-by-init-of-SocketListener4Xi to: " + str(ack2listen)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
            except:
                message2print = "{(Step 4/9d) yielded" + "; nack2listen-yielded-by-init-of-SocketListener4Xi: " + str(nack2listen)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                            
        else:
            message2print = "{(Step 4/9e) yielded" + "; nack2listen-yielded-by-init-of-SocketListener4Xi: " + str(nack2listen)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                   
                          
        # -----------------------------------------------------------------------------------------    
        # (Step 5/9)
        # -----------------------------------------------------------------------------------------                 
        ack2address = False
        nack2address = True
        
        if ((ack2socket is True) and (ack2address is True)):                  
            try:
                self._address = self._socket.getsockname()
                
                message2print = "{(Step 5/9a) set" + "; address-of-socketlistener2xi-set-by-init-of-SocketListener4Xi to: " + str(self._address)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                
                ack2address = True
                
                message2print = "{(Step 5/9b) set" + "; ack2address-set-by-init-of-SocketListener4Xi to: " + str(ack2address)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
            except:
                message2print = "{(Step 5/9c) yielded" + "; nack2address-yielded-by-init-of-SocketListener4Xi: " + str(nack2address)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                       
     
        else:
            message2print = "{(Step 5/9d) yielded" + "; nack2address-yielded-by-init-of-SocketListener4Xi: " + str(nack2address)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                
                          
        # -----------------------------------------------------------------------------------------    
        # (Step 6/9)
        # -----------------------------------------------------------------------------------------                 
        ack2family = False
        nack2family = True
        try:      
            self._family = family
            
            message2print = "{(Step 6/9a) set" + "; family-of-socketlistener2xi-set-by-init-of-SocketListener4Xi to: " + str(self._family)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
        
            ack2family = True 
            
            message2print = "{(Step 6/9b) set" + "; ack2family-set-by-init-of-SocketListener4Xi to: " + str(ack2family)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
        except:
            message2print = "{(Step 6/9c) yielded" + "; nack2family-yielded-by-init-of-SocketListener4Xi: " + str(nack2family)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            
              
                          
        # -----------------------------------------------------------------------------------------    
        # (Step 7/9)
        # -----------------------------------------------------------------------------------------                 
        ack2accepted = False
        nack2accepted = True
        
        try:      
            self._last_accepted = None
            
            message2print = "{(Step 7/9a) set" + "; last-accepted-of-socketlistener2xi-set-by-init-of-SocketListener4Xi to: " + str(self._last_accepted)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
        
            ack2accepted = True   

            message2print = "{(Step 7/9b) set" + "; ack2accepted-set-by-init-of-SocketListener4Xi to: " + str(ack2accepted)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
        except:
            message2print = "{(Step 7/9c) yielded" + "; nack2accepted-yielded-by-init-of-SocketListener4Xi: " + str(nack2accepted)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)              
            
    
        # -----------------------------------------------------------------------------------------    
        # (Step 8/9)
        # -----------------------------------------------------------------------------------------                 
        ack2unlink = False
        nack2unlink = True
        try:
            if family == get_af_unix4xi():
            
                self._unlink = Finalize4Xi(self, os.unlink, args=(address,), exitpriority=0)
                
                message2print = "{(Step 8/9a) set" + "; unlink-of-socketlistener2xi-set-by-init-of-SocketListener4Xi to: " + str(self._unlink)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
                                
                ack2unlink = True
                
                message2print = "{(Step 8/9b) set" + "; ack2unlink-set-by-init-of-SocketListener4Xi to: " + str(ack2unlink)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
                
                                    
            else:
                self._unlink = None
                
                message2print = "{(Step 8/9c) set" + "; unlink-of-socketlistener2xi-set-by-init-of-SocketListener4Xi to: " + str(self._unlink)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                
                message2print = "{(Step 8/9d) yielded" + "; nack2unlink-yielded-by-init-of-SocketListener4Xi: " + str(nack2unlink)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)      
                   
        except:
            message2print = "{(Step 8/9e) yielded" + "; nack2unlink-yielded-by-init-of-SocketListener4Xi: " + str(nack2unlink)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)           
                 
        
        # -----------------------------------------------------------------------------------------    
        # (Step 9/9)
        # -----------------------------------------------------------------------------------------  
        
        message2print = "{(Step 9/9a) inherited" + "; object-inherited-by-socketlistener2xi-of-SocketListener4Xi: " + str(object)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
                
        message2print = "{(Step 9/9b) activated" + "; socketlistener2xi-of-SocketListener4Xi-activated: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
                         
        message2print = "{(Step 9/9c) returning" + "; init-of-SocketListener4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                        
                
                        
    #</methodize4xi>
                  
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "accept-of-SocketListener4Xi" usage = "accepts">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>      
    # ---------------------------------------------------------------------------------------------
    def accept(self):
        '''
        SocketListener4Xi method accept: accepts
        ''' 
             
        # -----------------------------------------------------------------------------------------    
        # (Step 1/6)
        # -----------------------------------------------------------------------------------------                 
           
        class2print = "SocketListener4Xi"
        method2print = "accept"
        
        message2print = "{(Step 1/6) started" + "; accept-of-SocketListener4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
             
        # -----------------------------------------------------------------------------------------    
        # (Step 2/6)
        # -----------------------------------------------------------------------------------------                 
        message2print = "{(Step 2/6a) calling" + "; socket-accept-of-socketlistener2xi-calling-by-accept-of-SocketListener4Xi: " + str(self._socket.accept) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)       
     
        try:
            s2xi, self._last_accepted = self._socket.accept()
            
            message2print = "{(Step 2/6b) set" + "; s2xi-set-by-accept-of-SocketListener4Xi to: " + str(s2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
                            
            
            message2print = "{(Step 2/6c) set" + "; last-accepted-of-socketlistener2xi-set-by-accept-of-SocketListener4Xi to: " + str(self._last_accepted)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
        except:
            message2print = "{(Step 2/6d) yielded" + "; socket-accept-yielded-by-accept-of-SocketListener4Xi: " + str(self._socket.accept)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
                                   
        
        # -----------------------------------------------------------------------------------------    
        # (Step 3/6)
        # -----------------------------------------------------------------------------------------                 
        message2print = "{(Step 3/6a) passing" + "; fileno-passing-to-duplicate4xi-by-accept-of-SocketListener4Xi: " + str(s2xi.fileno()) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
                        
        try:
            fd2xi = duplicate4xi(s2xi.fileno())
            
            message2print = "{(Step 3/6b) set" + "; fd2xi-set-by-accept-of-SocketListener4Xi to: " + str(fd2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
        except:
            message2print = "{(Step 3/6c) yielded" + "; duplicate4xi-yielded-by-accept-of-SocketListener4Xi: " + str(duplicate4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
                        
                              
        
        # -----------------------------------------------------------------------------------------    
        # (Step 4/6)
        # -----------------------------------------------------------------------------------------                 
        message2print = "{(Step 4/6a) passing" + "; fd2xi-passing-to-Connection-by-accept-of-SocketListener4Xi: " + str(fd2xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            

        try:   
            conn2xi = Connection(fd2xi) # THIS MUST BE AS IT IS TO AVOID CYCLIC IMPORTS!
            
            message2print = "{(Step 4/6b) set" + "; conn2xi-set-by-accept-of-SocketListener4Xi to: " + str(conn2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
        except:
            message2print = "{(Step 4/6c) yielded" + "; multiprocessing-Connection-yielded-by-accept-of-SocketListener4Xi: " + str(Connection)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            
                        
            
        
        # -----------------------------------------------------------------------------------------    
        # (Step 5/6)
        # -----------------------------------------------------------------------------------------                 
        try:   
            s2xi.close()
            
            message2print = "{(Step 5/6a) closed" + "; s2xi-closed-by-accept-of-SocketListener4Xi: " + str(s2xi.close) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                   
        except:
            message2print = "{(Step 5/6b) yielded" + "; close-yielded-by-accept-of-SocketListener4Xi: " + str(s2xi.close)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
                         
        
        # -----------------------------------------------------------------------------------------    
        # (Step 6/6)
        # -----------------------------------------------------------------------------------------                 
        message2print = "{Step 6/6) returning" + "; connection-returning-by-accept-of-SocketListener4Xi: " + str(conn2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
                                
                   
        return conn2xi
    #</methodize4xi>
            
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "close-of-SocketListener4Xi" usage = "closes">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>      
    # ---------------------------------------------------------------------------------------------
    def close(self):
        '''
        SocketListener4Xi method close: closes
        ''' 
        
        # -----------------------------------------------------------------------------------------    
        # (Step 1/4)
        # -----------------------------------------------------------------------------------------                 
            
        class2print = "SocketListener4Xi"
        method2print = "close"
        
        message2print = "{started" + "; close-of-SocketListener4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
           
        
        # -----------------------------------------------------------------------------------------    
        # (Step 2/4)
        # -----------------------------------------------------------------------------------------                 
        try:   
            self._socket.close()
            
            message2print = "{(Step 2/4a) performed" + "; socket-close-of-socketlistener2xi-performed-by-close-of-SocketListener4Xi: " + str(self._socket.close) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
        except:
            message2print = "{(Step 2/4b) yielded" + "; close-yielded-by-close-of-SocketListener4Xi: " + str(self._socket.close)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
                   
        
        # -----------------------------------------------------------------------------------------    
        # (Step 3/4)
        # -----------------------------------------------------------------------------------------                 
                   
        if self._unlink is not None:
            self._unlink()
            
                    
            message2print = "{(Step 3/4) performed" + "; unlink-of-socketlistener2xi-performed-by-close-of-SocketListener4Xi: " + str(self._unlink) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
   
            
        
        
        # -----------------------------------------------------------------------------------------    
        # (Step 4/4)
        # -----------------------------------------------------------------------------------------
 
        message2print = "{(Step 4/4) returning" + "; close-of-SocketListener4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        
        return                        
                        
                                
    #</methodize4xi>
                
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "sense-of-SocketListener4Xi" usage = "senses SocketListener4Xi class objects">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>      
    # ---------------------------------------------------------------------------------------------
    def sense(self):
        '''
        senses SocketListener4Xi class objects
        '''

        
        class2print = 'SocketListener4Xi'
        method2print = 'sense'
        
        
        message2print = "{started" + "; sense-of-SocketListener4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
   
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------  
        message2print = "{sensing" + "; SocketListener4Xi-sensing-by-sense-of-SocketListener4Xi: " + str(SocketListener4Xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
                        
        
        # -----------------------------------------------------------------------------------------
        # get sensed knowledge for this object
        # -----------------------------------------------------------------------------------------        
        kb2xi = sense_object4xi(self)
        
        message2print = "{got" + "; kb2xi-got-by-sense-of-SocketListener4Xi: " + str(kb2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
                
        
        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # -----------------------------------------------------------------------------------------         
        try:
            kb2xi['socket-of-socketlistener2xi'] = self._socket
            kb2xi['socket-bind-of-socketlistener2xi'] = self._socket.bind                 
            kb2xi['socket-listen-of-socketlistener2xi'] = self._socket.listen
            kb2xi['address-of-socketlistener2xi'] = self._address          
            kb2xi['family-of-socketlistener2xi'] = self._family
            kb2xi['last-accepted-of-socketlistener2xi'] = self._last_accepted                 
            kb2xi['unlink-of-socketlistener2xi'] = self._unlink
            kb2xi['socket-close-of-socketlistener2xi'] = self._socket.close 

        except:
            message2print = "{yielded" + "; kb2xi-yielded-by-sense-of-SocketListener4Xi: " + str(kb2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            

        message2print = "{sensed" + "; SocketListener4Xi-sensed-by-sense-of-SocketListener4Xi: " + str(SocketListener4Xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                            
        # -----------------------------------------------------------------------------------------
        # 
        # -----------------------------------------------------------------------------------------         
        message2print = "{returning" + "; kb2xi-returning-by-sense-of-SocketListener4Xi: " + str(kb2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
                            
          
        return kb2xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
    #</methodize4xi>
    
        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "reason-of-SocketListener4X" usage = "reasons SocketListener4Xi class objects">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>  
    # ---------------------------------------------------------------------------------------------
    def reason(self):
        '''
        reasons SocketListener4Xi class objects
        '''
  
        
        class2print = 'SocketListener4Xi'
        method2print = 'reason'

        
        message2print = "{started" + "; reason-of-SocketListener4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
         
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------
        message2print = "{reasoning" + "; SocketListener4Xi-reasoning-by-reason-of-SocketListener4Xi: " + str(SocketListener4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
                
        
        # -----------------------------------------------------------------------------------------
        # get reasoned knowledge for this object
        # -----------------------------------------------------------------------------------------         
        kb4xi = reason_object4xi(self)
        
        
        message2print = "{got" + "; kb4xi-got-by-reason-of-SocketListener4Xi: " + str(kb4xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
                
        
        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # format example: kb4xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------          
        
        
          
        
        message2print = "{reasoned" + "; SocketListener4Xi-reasoned-by-reason-of-SocketListener4Xi: " + str(SocketListener4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)     
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
        message2print = "{returning" + "; kb4xi-returning-by-reason-of-SocketListener4Xi: " + str(kb4xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
                           
          
        return kb4xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------            
    #</methodize4xi>
    
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "ace-of-SocketListener4Xi" usage = "aces SocketListener4Xi class objects">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>  
    # ---------------------------------------------------------------------------------------------
    def ace(self):
        '''
        aces SocketListener4Xi class objects 
        '''
    
        
        
        class2print = 'SocketListener4Xi'
        method2print = 'ace'
        
        
        message2print = "{started" + "; ace-of-SocketListener4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
            
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------
        message2print = "{acing" + "; SocketListener4Xi-acing-by-ace-of-SocketListener4Xi: " + str(SocketListener4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
                    
        
        # -----------------------------------------------------------------------------------------
        # get aced knowledge for this object
        # -----------------------------------------------------------------------------------------        
        kb8xi = ace_object4xi(self)  
        
        message2print = "{got" + "; kb8xi-got-by-ace-of-SocketListener4Xi: " + str(kb8xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
                


        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # format example: kb8xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------              
        
          
        message2print = "{aced" + "; SocketListener4Xi-aced-by-ace-of-SocketListener4Xi: " + str(SocketListener4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
        message2print = "{returning" + "; kb8xi-returning-by-ace-of-SocketListener4Xi: " + str(kb8xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
                         
        
        return kb8xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------            
    #</methodize4xi>
#</class4xi>
#</program4xi>

# -------------------------------------------------------------------------------------------------
# Recorded History                             
# ------------------------------------------------------------------------------------------------- 
#<record4xi record = "
#   test_none4xi.txt
#   sense-test-none4xi.xml
#   reason-test-none4xi.xml
#   ace-test-none4xi.xml
#"
#>

# -------------------------------------------------------------------------------------------------
# Legal Notice                             
# ------------------------------------------------------------------------------------------------- 
#<protect4xi protection = "copyright"
#   owner = "Small & Smart"
#   rights = "All rights reserved"
#   state = "California"
#   country = "United States of America"
#   references = "Legal notice at www.dragonxi.com; www.yourdragonxi.com; www.yourdragonxi.org; www.dragon2xi.com/; www.dragon4xi.com/; www.dragon8xi.com/ " 
#   codes = "Codes established in California and applicable to corporations in California"
#   compliance = "The California Law"
#   court = "Supreme Court of California, 350 McAllister Street, San Francisco, CA 94102-4783"
#/>
 

#<end4xi end = "
# -------------------------------------------------------------------------------------------------
# end of socketlistener4xi.py                             
# ------------------------------------------------------------------------------------------------- 
#"
#/>

